@@ -320,36 +320,8 @@ static void DbugVfprintf(FILE *stream, const char* format, va_list args);
 */
 
 #include <my_pthread.h>
-/*
-** Protects writing to all file descriptors, init_settings.keywords
-** pointer and it's pointee - a linked list with keywords.
-*/
 static pthread_mutex_t THR_LOCK_dbug;
 
-static void LockMutex(CODE_STATE *cs)
-{
-  if (!cs->locked)
-    pthread_mutex_lock(&THR_LOCK_dbug);
-  cs->locked++;
-}
-static void UnlockMutex(CODE_STATE *cs)
-{
-  --cs->locked;
-  assert(cs->locked >= 0);
-  if (cs->locked == 0)
-    pthread_mutex_unlock(&THR_LOCK_dbug);
-}
-static void LockIfInitSettings(CODE_STATE *cs)
-{
-  if (cs->stack == &init_settings)
-    LockMutex(cs);
-}
-static void UnlockIfInitSettings(CODE_STATE *cs)
-{
-  if (cs->stack == &init_settings)
-    UnlockMutex(cs);
-}
-
 static CODE_STATE *code_state(void)
 {
   CODE_STATE *cs, **cs_ptr;
@@ -477,16 +449,22 @@ static int DbugParse(CODE_STATE *cs, const char *control)
   const char *end;
   int rel, f_used=0;
   struct settings *stack;
+  int org_cs_locked;
 
   stack= cs->stack;
 
+  if (!(org_cs_locked= cs->locked))
+  {
+    cs->locked= 1;
+    pthread_mutex_lock(&THR_LOCK_dbug);
+  }
+
   if (control[0] == '-' && control[1] == '#')
     control+=2;
 
   rel= control[0] == '+' || control[0] == '-';
   if ((!rel || (!stack->out_file && !stack->next)))
   {
-    LockIfInitSettings(cs);
     FreeState(cs, 0);
     stack->flags= 0;
     stack->delay= 0;
@@ -496,7 +474,6 @@ static int DbugParse(CODE_STATE *cs, const char *control)
     stack->functions= NULL;
     stack->keywords= NULL;
     stack->processes= NULL;
-    UnlockIfInitSettings(cs);
   }
   else if (!stack->out_file)
   {
@@ -511,9 +488,7 @@ static int DbugParse(CODE_STATE *cs, const char *control)
     {
       /* never share with the global parent - it can change under your feet */
       stack->functions= ListCopy(init_settings.functions);
-      LockIfInitSettings(cs);
       stack->keywords= ListCopy(init_settings.keywords);
-      UnlockIfInitSettings(cs);
       stack->processes= ListCopy(init_settings.processes);
     }
     else
@@ -537,31 +512,21 @@ static int DbugParse(CODE_STATE *cs, const char *control)
     case 'd':
       if (sign < 0 && control == end)
       {
-        LockIfInitSettings(cs);
         if (!is_shared(stack, keywords))
           FreeList(stack->keywords);
         stack->keywords=NULL;
-        UnlockIfInitSettings(cs);
         stack->flags &= ~DEBUG_ON;
         break;
       }
-      LockIfInitSettings(cs);
       if (rel && is_shared(stack, keywords))
         stack->keywords= ListCopy(stack->keywords);
-      UnlockIfInitSettings(cs);
       if (sign < 0)
       {
         if (DEBUGGING)
-        {
-          LockIfInitSettings(cs);
           stack->keywords= ListDel(stack->keywords, control, end);
-          UnlockIfInitSettings(cs);
-        }
         break;
       }
-      LockIfInitSettings(cs);
       stack->keywords= ListAdd(stack->keywords, control, end);
-      UnlockIfInitSettings(cs);
       stack->flags |= DEBUG_ON;
       break;
     case 'D':
@@ -696,6 +661,11 @@ static int DbugParse(CODE_STATE *cs, const char *control)
     control=end+1;
     end= DbugStrTok(control);
   }
+  if (!org_cs_locked)
+  {
+    pthread_mutex_unlock(&THR_LOCK_dbug);
+    cs->locked= 0;
+  }
   return !rel || f_used;
 }
   
@@ -1028,9 +998,7 @@ int _db_explain_ (CODE_STATE *cs, char *buf, size_t len)
 
   get_code_state_if_not_set_or_return *buf=0;
 
-  LockIfInitSettings(cs);
   op_list_to_buf('d', cs->stack->keywords, DEBUGGING);
-  UnlockIfInitSettings(cs);
   op_int_to_buf ('D', cs->stack->delay, 0);
   op_list_to_buf('f', cs->stack->functions, cs->stack->functions);
   op_bool_to_buf('F', cs->stack->flags & FILE_ON);
@@ -1125,6 +1093,7 @@ int _db_explain_init_(char *buf, size_t len)
 void _db_enter_(const char *_func_, const char *_file_,
                 uint _line_, struct _db_stack_frame_ *_stack_frame_)
 {
+  int save_errno;
   CODE_STATE *cs;
   if (!((cs=code_state())))
   {
@@ -1132,6 +1101,7 @@ void _db_enter_(const char *_func_, const char *_file_,
     _stack_frame_->prev= 0;
     return;
   }
+  save_errno= errno;
 
   _stack_frame_->line= -1;
   _stack_frame_->func= cs->func;
@@ -1152,14 +1122,12 @@ void _db_enter_(const char *_func_, const char *_file_,
       cs->stack->flags &= ~SANITY_CHECK_ON;
     if (TRACING)
     {
-      int save_errno= errno;
-      LockMutex(cs);
+      if (!cs->locked)
+        pthread_mutex_lock(&THR_LOCK_dbug);
       DoPrefix(cs, _line_);
       Indent(cs, cs->level);
       (void) fprintf(cs->stack->out_file->file, ">%s\n", cs->func);
-      UnlockMutex(cs);
-      DbugFlush(cs);
-      errno=save_errno;
+      DbugFlush(cs);                       /* This does a unlock */
     }
     break;
   case DISABLE_TRACE:
@@ -1168,6 +1136,7 @@ void _db_enter_(const char *_func_, const char *_file_,
   case DONT_TRACE:
     break;
   }
+  errno=save_errno;
 }
 
 /*
@@ -1192,6 +1161,7 @@ void _db_enter_(const char *_func_, const char *_file_,
 
 void _db_return_(struct _db_stack_frame_ *_stack_frame_)
 {
+  int save_errno=errno;
   uint _slevel_= _stack_frame_->level & ~TRACE_ON;
   CODE_STATE *cs;
   get_code_state_or_return;
@@ -1212,14 +1182,12 @@ void _db_return_(struct _db_stack_frame_ *_stack_frame_)
       cs->stack->flags &= ~SANITY_CHECK_ON;
     if (TRACING)
     {
-      int save_errno=errno;
-      LockMutex(cs);
+      if (!cs->locked)
+        pthread_mutex_lock(&THR_LOCK_dbug);
       DoPrefix(cs, _stack_frame_->line);
       Indent(cs, cs->level);
       (void) fprintf(cs->stack->out_file->file, "<%s\n", cs->func);
-      UnlockMutex(cs);
       DbugFlush(cs);
-      errno=save_errno;
     }
   }
   /*
@@ -1231,6 +1199,7 @@ void _db_return_(struct _db_stack_frame_ *_stack_frame_)
   cs->file= _stack_frame_->file;
   if (cs->framep != NULL)
     cs->framep= cs->framep->prev;
+  errno=save_errno;
 }
 
 
@@ -1295,24 +1264,27 @@ void _db_doprnt_(const char *format,...)
   CODE_STATE *cs;
   get_code_state_or_return;
 
+  va_start(args,format);
 
+  if (!cs->locked)
+    pthread_mutex_lock(&THR_LOCK_dbug);
   if (_db_keyword_(cs, cs->u_keyword, 0))
   {
     int save_errno=errno;
-    LockMutex(cs);
     DoPrefix(cs, cs->u_line);
     if (TRACING)
       Indent(cs, cs->level + 1);
     else
       (void) fprintf(cs->stack->out_file->file, "%s: ", cs->func);
     (void) fprintf(cs->stack->out_file->file, "%s: ", cs->u_keyword);
-    va_start(args,format);
     DbugVfprintf(cs->stack->out_file->file, format, args);
-    UnlockMutex(cs);
-    va_end(args);
     DbugFlush(cs);
     errno=save_errno;
   }
+  else if (!cs->locked)
+    pthread_mutex_unlock(&THR_LOCK_dbug);
+
+  va_end(args);
 }
 
 /*
@@ -1353,9 +1325,10 @@ void _db_dump_(uint _line_, const char *keyword,
   CODE_STATE *cs;
   get_code_state_or_return;
 
+  if (!cs->locked)
+    pthread_mutex_lock(&THR_LOCK_dbug);
   if (_db_keyword_(cs, keyword, 0))
   {
-    LockMutex(cs);
     DoPrefix(cs, _line_);
     if (TRACING)
     {
@@ -1383,9 +1356,10 @@ void _db_dump_(uint _line_, const char *keyword,
       fputc(' ',cs->stack->out_file->file);
     }
     (void) fputc('\n',cs->stack->out_file->file);
-    UnlockMutex(cs);
     DbugFlush(cs);
   }
+  else if (!cs->locked)
+    pthread_mutex_unlock(&THR_LOCK_dbug);
 }
 
 
@@ -1611,10 +1585,8 @@ static void PushState(CODE_STATE *cs)
 static void FreeState(CODE_STATE *cs, int free_state)
 {
   struct settings *state= cs->stack;
-  LockIfInitSettings(cs);
   if (!is_shared(state, keywords))
     FreeList(state->keywords);
-  UnlockIfInitSettings(cs);
   if (!is_shared(state, functions))
     FreeList(state->functions);
   if (!is_shared(state, processes))
@@ -1737,16 +1709,10 @@ FILE *_db_fp_(void)
 BOOLEAN _db_keyword_(CODE_STATE *cs, const char *keyword, int strict)
 {
   int match= strict ? INCLUDE : INCLUDE|MATCHED;
-  BOOLEAN result = FALSE;
   get_code_state_if_not_set_or_return FALSE;
 
-  if (!DEBUGGING || !(DoTrace(cs) & DO_TRACE))
-    return FALSE;
-
-  LockIfInitSettings(cs);
-  result= (InList(cs->stack->keywords, keyword, strict) & match) ? TRUE : FALSE;
-  UnlockIfInitSettings(cs);
-  return result;
+  return (DEBUGGING && DoTrace(cs) & DO_TRACE &&
+          InList(cs->stack->keywords, keyword, strict) & match);
 }
 
 /*
@@ -1972,16 +1938,16 @@ static void DBUGCloseFile(CODE_STATE *cs, sFILE *new_value)
   sFILE *fp;
   if (!cs || !cs->stack || !cs->stack->out_file)
     return;
+  if (!cs->locked)
+    pthread_mutex_lock(&THR_LOCK_dbug);
 
   fp= cs->stack->out_file;
   if (--fp->used == 0)
   {
     if (fclose(fp->file) == EOF)
     {
-      LockMutex(cs);
       (void) fprintf(stderr, ERR_CLOSE, cs->process);
       perror("");
-      UnlockMutex(cs);
     }
     else
     {
@@ -1989,6 +1955,8 @@ static void DBUGCloseFile(CODE_STATE *cs, sFILE *new_value)
     }
   }
   cs->stack->out_file= new_value;
+  if (!cs->locked)
+    pthread_mutex_unlock(&THR_LOCK_dbug);
 }
 
 
@@ -2160,6 +2128,8 @@ static void DbugFlush(CODE_STATE *cs)
     if (cs->stack->delay)
       (void) Delay(cs->stack->delay);
   }
+  if (!cs->locked)
+    pthread_mutex_unlock(&THR_LOCK_dbug);
 } /* DbugFlush */
 
 
